@@ -1,4 +1,4 @@
-// Copyright (C) 2004-2021 Artifex Software, Inc.
+// Copyright (C) 2004-2025 Artifex Software, Inc.
 //
 // This file is part of MuPDF.
 //
@@ -17,8 +17,8 @@
 //
 // Alternative licensing terms are available from the licensor.
 // For commercial licensing, see <https://www.artifex.com/> or contact
-// Artifex Software, Inc., 1305 Grant Avenue - Suite 200, Novato,
-// CA 94945, U.S.A., +1(415)492-9861, for further information.
+// Artifex Software, Inc., 39 Mesa Street, Suite 108A, San Francisco,
+// CA 94129, USA, for further information.
 
 #ifndef MUDPF_FITZ_H
 #define MUDPF_FITZ_H
@@ -43,6 +43,7 @@ extern "C" {
 #include "mupdf/fitz/tree.h"
 #include "mupdf/fitz/bidi.h"
 #include "mupdf/fitz/xml.h"
+#include "mupdf/fitz/json.h"
 
 /* I/O */
 #include "mupdf/fitz/buffer.h"
@@ -51,13 +52,15 @@ extern "C" {
 #include "mupdf/fitz/compressed-buffer.h"
 #include "mupdf/fitz/filter.h"
 #include "mupdf/fitz/archive.h"
+#include "mupdf/fitz/heap.h"
+
 
 /* Resources */
 #include "mupdf/fitz/store.h"
 #include "mupdf/fitz/color.h"
 #include "mupdf/fitz/pixmap.h"
-#include "mupdf/fitz/bitmap.h"
 #include "mupdf/fitz/image.h"
+#include "mupdf/fitz/bitmap.h"
 #include "mupdf/fitz/shade.h"
 #include "mupdf/fitz/font.h"
 #include "mupdf/fitz/path.h"
@@ -85,6 +88,12 @@ extern "C" {
 #include "mupdf/fitz/write-pixmap.h"
 #include "mupdf/fitz/output-svg.h"
 
+#include "mupdf/fitz/story.h"
+#include "mupdf/fitz/story-writer.h"
+
+#include "mupdf/fitz/deskew.h"
+#include "mupdf/fitz/barcode.h"
+
 #ifdef __cplusplus
 }
 #endif
