@@ -1,4 +1,4 @@
-// Copyright (C) 2004-2023 Artifex Software, Inc.
+// Copyright (C) 2004-2025 Artifex Software, Inc.
 //
 // This file is part of MuPDF.
 //
@@ -101,7 +101,7 @@ void fz_pcl_preset(fz_context *ctx, fz_pcl_options *opts, const char *preset);
 fz_pcl_options *fz_parse_pcl_options(fz_context *ctx, fz_pcl_options *opts, const char *args);
 
 /**
-	Create a new band writer, outputing monochrome pcl.
+	Create a new band writer, outputting monochrome pcl.
 */
 fz_band_writer *fz_new_mono_pcl_band_writer(fz_context *ctx, fz_output *out, const fz_pcl_options *options);
 
@@ -116,7 +116,7 @@ void fz_write_bitmap_as_pcl(fz_context *ctx, fz_output *out, const fz_bitmap *bi
 void fz_save_bitmap_as_pcl(fz_context *ctx, fz_bitmap *bitmap, char *filename, int append, const fz_pcl_options *pcl);
 
 /**
-	Create a new band writer, outputing color pcl.
+	Create a new band writer, outputting color pcl.
 */
 fz_band_writer *fz_new_color_pcl_band_writer(fz_context *ctx, fz_output *out, const fz_pcl_options *options);
 
@@ -154,7 +154,7 @@ typedef struct
 fz_pclm_options *fz_parse_pclm_options(fz_context *ctx, fz_pclm_options *opts, const char *args);
 
 /**
-	Create a new band writer, outputing pclm
+	Create a new band writer, outputting pclm
 */
 fz_band_writer *fz_new_pclm_band_writer(fz_context *ctx, fz_output *out, const fz_pclm_options *options);
 
@@ -166,7 +166,7 @@ void fz_write_pixmap_as_pclm(fz_context *ctx, fz_output *out, const fz_pixmap *p
 /**
 	Save a (Greyscale or RGB) pixmap as pclm.
 */
-void fz_save_pixmap_as_pclm(fz_context *ctx, fz_pixmap *pixmap, char *filename, int append, const fz_pclm_options *options);
+void fz_save_pixmap_as_pclm(fz_context *ctx, fz_pixmap *pixmap, const char *filename, int append, const fz_pclm_options *options);
 
 /**
 	PDFOCR output
@@ -178,6 +178,10 @@ typedef struct
 	char language[256];
 	char datadir[1024];
 
+	int skew_correct; /* 0 = no skew correction. 1 = automatic. 2 = use specified angle. */
+	float skew_angle; /* Only used if skew == 2 */
+	int skew_border; /* 0 = increase size so no content is lost. 1 = maintain size. 2 = decrease size so no new pixels are visible. */
+
 	/* Updated as we move through the job */
 	int page_count;
 } fz_pdfocr_options;
@@ -196,7 +200,7 @@ typedef struct
 fz_pdfocr_options *fz_parse_pdfocr_options(fz_context *ctx, fz_pdfocr_options *opts, const char *args);
 
 /**
-	Create a new band writer, outputing pdfocr.
+	Create a new band writer, outputting pdfocr.
 
 	Ownership of output stays with the caller, the band writer
 	borrows the reference. The caller must keep the output around
@@ -263,11 +267,13 @@ void fz_save_pixmap_as_jpx(fz_context *ctx, fz_pixmap *pixmap, const char *filen
 fz_band_writer *fz_new_png_band_writer(fz_context *ctx, fz_output *out);
 
 /**
-	Reencode a given image as a PNG into a buffer.
+	Re-encode a given image as a PNG into a buffer.
 
 	Ownership of the buffer is returned.
 */
 fz_buffer *fz_new_buffer_from_image_as_png(fz_context *ctx, fz_image *image, fz_color_params color_params);
+fz_buffer *fz_new_buffer_from_image_as_pbm(fz_context *ctx, fz_image *image, fz_color_params color_params);
+fz_buffer *fz_new_buffer_from_image_as_pkm(fz_context *ctx, fz_image *image, fz_color_params color_params);
 fz_buffer *fz_new_buffer_from_image_as_pnm(fz_context *ctx, fz_image *image, fz_color_params color_params);
 fz_buffer *fz_new_buffer_from_image_as_pam(fz_context *ctx, fz_image *image, fz_color_params color_params);
 fz_buffer *fz_new_buffer_from_image_as_psd(fz_context *ctx, fz_image *image, fz_color_params color_params);
@@ -275,11 +281,13 @@ fz_buffer *fz_new_buffer_from_image_as_jpeg(fz_context *ctx, fz_image *image, fz
 fz_buffer *fz_new_buffer_from_image_as_jpx(fz_context *ctx, fz_image *image, fz_color_params color_params, int quality);
 
 /**
-	Reencode a given pixmap as a PNG into a buffer.
+	Re-encode a given pixmap as a PNG into a buffer.
 
 	Ownership of the buffer is returned.
 */
 fz_buffer *fz_new_buffer_from_pixmap_as_png(fz_context *ctx, fz_pixmap *pixmap, fz_color_params color_params);
+fz_buffer *fz_new_buffer_from_pixmap_as_pbm(fz_context *ctx, fz_pixmap *pixmap, fz_color_params color_params);
+fz_buffer *fz_new_buffer_from_pixmap_as_pkm(fz_context *ctx, fz_pixmap *pixmap, fz_color_params color_params);
 fz_buffer *fz_new_buffer_from_pixmap_as_pnm(fz_context *ctx, fz_pixmap *pixmap, fz_color_params color_params);
 fz_buffer *fz_new_buffer_from_pixmap_as_pam(fz_context *ctx, fz_pixmap *pixmap, fz_color_params color_params);
 fz_buffer *fz_new_buffer_from_pixmap_as_psd(fz_context *ctx, fz_pixmap *pix, fz_color_params color_params);
@@ -297,7 +305,7 @@ void fz_save_pixmap_as_pnm(fz_context *ctx, fz_pixmap *pixmap, const char *filen
 void fz_write_pixmap_as_pnm(fz_context *ctx, fz_output *out, fz_pixmap *pixmap);
 
 /**
-	Create a band writer targetting pnm (greyscale or rgb, no
+	Create a band writer targeting pnm (greyscale or rgb, no
 	alpha).
 */
 fz_band_writer *fz_new_pnm_band_writer(fz_context *ctx, fz_output *out);
@@ -315,7 +323,7 @@ void fz_save_pixmap_as_pam(fz_context *ctx, fz_pixmap *pixmap, const char *filen
 void fz_write_pixmap_as_pam(fz_context *ctx, fz_output *out, fz_pixmap *pixmap);
 
 /**
-	Create a band writer targetting pnm (greyscale, rgb or cmyk,
+	Create a band writer targeting pnm (greyscale, rgb or cmyk,
 	with or without alpha).
 */
 fz_band_writer *fz_new_pam_band_writer(fz_context *ctx, fz_output *out);
@@ -331,7 +339,7 @@ void fz_save_bitmap_as_pbm(fz_context *ctx, fz_bitmap *bitmap, const char *filen
 void fz_write_bitmap_as_pbm(fz_context *ctx, fz_output *out, fz_bitmap *bitmap);
 
 /**
-	Create a new band writer, targetting pbm.
+	Create a new band writer, targeting pbm.
 */
 fz_band_writer *fz_new_pbm_band_writer(fz_context *ctx, fz_output *out);
 
