@@ -1,4 +1,4 @@
-// Copyright (C) 2004-2023 Artifex Software, Inc.
+// Copyright (C) 2004-2025 Artifex Software, Inc.
 //
 // This file is part of MuPDF.
 //
@@ -35,13 +35,34 @@ pdf_processor *pdf_keep_processor(fz_context *ctx, pdf_processor *proc);
 void pdf_close_processor(fz_context *ctx, pdf_processor *proc);
 void pdf_drop_processor(fz_context *ctx, pdf_processor *proc);
 
+/*
+ * Stack of resources used during interpretation (holding page and
+ * xobject/font/type3 glyph resource dictionaries).
+ */
+
+struct pdf_resource_stack
+{
+	pdf_resource_stack *next;
+	pdf_obj *resources;
+};
+
+pdf_obj *pdf_lookup_resource(fz_context *ctx, pdf_resource_stack *stack, pdf_obj *type, const char *name);
+
+typedef enum
+{
+	PDF_PROCESSOR_REQUIRES_DECODED_IMAGES = 1
+} pdf_processor_requirements;
+
 struct pdf_processor
 {
 	int refs;
 
+	int closed;
+
 	/* close the processor. Also closes any chained processors. */
 	void (*close_processor)(fz_context *ctx, pdf_processor *proc);
 	void (*drop_processor)(fz_context *ctx, pdf_processor *proc);
+	void (*reset_processor)(fz_context *ctx, pdf_processor *proc);
 
 	/* At any stage, we can have one set of resources in place.
 	 * This function gives us a set of resources to use. We remember
@@ -50,9 +71,12 @@ struct pdf_processor
 	 * is to pass either it, or a filtered version of it onto any
 	 * chained processor. */
 	void (*push_resources)(fz_context *ctx, pdf_processor *proc, pdf_obj *res);
+
 	/* Pop the resources stack. This must be passed on to any chained
 	 * processors. This returns a pointer to the resource dict just
-	 * popped by the deepest filter. The caller inherits this reference. */
+	 * popped by the deepest filter. The caller inherits this reference.
+	 * NOTE: This must NOT throw as it may be called during cleanup!
+	 */
 	pdf_obj *(*pop_resources)(fz_context *ctx, pdf_processor *proc);
 
 	/* general graphics state */
@@ -68,7 +92,7 @@ struct pdf_processor
 	void (*op_gs_BM)(fz_context *ctx, pdf_processor *proc, const char *blendmode);
 	void (*op_gs_ca)(fz_context *ctx, pdf_processor *proc, float alpha);
 	void (*op_gs_CA)(fz_context *ctx, pdf_processor *proc, float alpha);
-	void (*op_gs_SMask)(fz_context *ctx, pdf_processor *proc, pdf_obj *smask, float *bc, int luminosity);
+	void (*op_gs_SMask)(fz_context *ctx, pdf_processor *proc, pdf_obj *smask, fz_colorspace *smask_cs, float *bc, int luminosity, pdf_obj *tr);
 	void (*op_gs_end)(fz_context *ctx, pdf_processor *proc);
 
 	/* special graphics state */
@@ -170,19 +194,27 @@ struct pdf_processor
 	void (*op_gs_OPM)(fz_context *ctx, pdf_processor *proc, int i);
 	void (*op_gs_UseBlackPtComp)(fz_context *ctx, pdf_processor *proc, pdf_obj *name);
 
+	/* EOD is used to signify end of data (before any finalise/close down/
+	 * automatically added gstate pops). */
+	void (*op_EOD)(fz_context *ctx, pdf_processor *proc);
+
 	/* END is used to signify end of stream (finalise and close down) */
 	void (*op_END)(fz_context *ctx, pdf_processor *proc);
 
 	/* interpreter state that persists across content streams */
 	const char *usage;
 	int hidden;
+
+	pdf_processor_requirements requirements;
+
+	/* resource dictionary stack */
+	pdf_resource_stack *rstack;
 };
 
 typedef struct
 {
 	/* input */
 	pdf_document *doc;
-	pdf_obj *rdb;
 	pdf_lexbuf *buf;
 	fz_cookie *cookie;
 
@@ -201,9 +233,11 @@ typedef struct
 	float stack[32];
 } pdf_csi;
 
+void pdf_count_q_balance(fz_context *ctx, pdf_document *doc, pdf_obj *res, pdf_obj *stm, int *prepend, int *append);
+
 /* Functions to set up pdf_process structures */
 
-pdf_processor *pdf_new_run_processor(fz_context *ctx, pdf_document *doc, fz_device *dev, fz_matrix ctm, int struct_parent, const char *usage, pdf_gstate *gstate, fz_default_colorspaces *default_cs, fz_cookie *cookie);
+pdf_processor *pdf_new_run_processor(fz_context *ctx, pdf_document *doc, fz_device *dev, fz_matrix ctm, int struct_parent, const char *usage, pdf_gstate *gstate, fz_default_colorspaces *default_cs, fz_cookie *cookie, pdf_gstate *fill_gstate, pdf_gstate *stroke_gstate);
 
 /*
 	Create a buffer processor.
@@ -215,8 +249,21 @@ pdf_processor *pdf_new_run_processor(fz_context *ctx, pdf_document *doc, fz_devi
 
 	ahxencode: If 0, then image streams will be send as binary,
 	otherwise they will be asciihexencoded.
+
+	newlines: If 0, then minimal spacing will be sent. If 1
+	then newlines will be sent after every operator.
 */
-pdf_processor *pdf_new_buffer_processor(fz_context *ctx, fz_buffer *buffer, int ahxencode);
+pdf_processor *pdf_new_buffer_processor(fz_context *ctx, fz_buffer *buffer, int ahxencode, int newlines);
+
+/*
+	Reopen a closed processor to be used again.
+
+	This brings a processor back to life after a close.
+	Not all processors may support this, so this may throw
+	an exception.
+*/
+void pdf_reset_processor(fz_context *ctx, pdf_processor *proc);
+
 
 /*
 	Create an output processor. This
@@ -226,8 +273,11 @@ pdf_processor *pdf_new_buffer_processor(fz_context *ctx, fz_buffer *buffer, int
 
 	ahxencode: If 0, then image streams will be send as binary,
 	otherwise they will be asciihexencoded.
+
+	newlines: If 0, then minimal spacing will be sent. If 1
+	then newlines will be sent after every operator.
 */
-pdf_processor *pdf_new_output_processor(fz_context *ctx, fz_output *out, int ahxencode);
+pdf_processor *pdf_new_output_processor(fz_context *ctx, fz_output *out, int ahxencode, int newlines);
 
 typedef struct pdf_filter_options pdf_filter_options;
 
@@ -276,6 +326,9 @@ typedef struct
 	Operators will be fed into the filter generated from the first
 	factory function in the list, and from there go to the filter
 	generated from the second factory in the list etc.
+
+	newlines: If 0, then minimal whitespace will be produced. If 1,
+	then a newline will be sent after every operator.
 */
 struct pdf_filter_options
 {
@@ -285,18 +338,23 @@ struct pdf_filter_options
 	int no_update;
 
 	void *opaque;
-	void (*complete)(fz_context *ctx, fz_buffer *buffer, void *arg);
+	void (*complete)(fz_context *ctx, fz_buffer *buffer, void *opaque);
 
 	pdf_filter_factory *filters;
+	int newlines;
 };
 
 typedef enum
 {
-	FZ_CULL_PATH_FILL,
-	FZ_CULL_PATH_STROKE,
-	FZ_CULL_PATH_FILL_STROKE,
-	FZ_CULL_CLIP_PATH,
-	FZ_CULL_GLYPH,
+	FZ_CULL_PATH_DROP = 0,
+	FZ_CULL_PATH_FILL = 1,
+	FZ_CULL_PATH_STROKE = 2,
+	FZ_CULL_PATH_FILL_STROKE = 3,
+	FZ_CULL_CLIP_PATH_DROP = 4,
+	FZ_CULL_CLIP_PATH_FILL = 5,
+	FZ_CULL_CLIP_PATH_STROKE = 6,
+	FZ_CULL_CLIP_PATH_FILL_STROKE = 7,
+	FZ_CULL_GLYPH = 8,
 	FZ_CULL_IMAGE,
 	FZ_CULL_SHADING
 } fz_cull_type;
@@ -385,13 +443,20 @@ pdf_obj *pdf_processor_pop_resources(fz_context *ctx, pdf_processor *proc);
 			*image either the same (for no change) or updated
 			to be a new one. Reference must be dropped, and a
 			new kept reference returned.
+
+	share_rewrite: function pointer called to rewrite a shade
+
+	repeated_image_rewrite: If 0, then each image is rewritten only once.
+		Otherwise, it is called for every instance (useful if gathering
+		information about the ctm).
 */
 typedef struct
 {
 	void *opaque;
 	void (*color_rewrite)(fz_context *ctx, void *opaque, pdf_obj **cs, int *n, float color[FZ_MAX_COLORS]);
-	void (*image_rewrite)(fz_context *ctx, void *opaque, fz_image **image);
+	void (*image_rewrite)(fz_context *ctx, void *opaque, fz_image **image, fz_matrix ctm, pdf_obj *obj);
 	pdf_shade_recolorer *shade_rewrite;
+	int repeated_image_rewrite;
 } pdf_color_filter_options;
 
 pdf_processor *
@@ -400,7 +465,7 @@ pdf_new_color_filter(fz_context *ctx, pdf_document *doc, pdf_processor *chain, i
 /*
 	Functions to actually process annotations, glyphs and general stream objects.
 */
-void pdf_process_contents(fz_context *ctx, pdf_processor *proc, pdf_document *doc, pdf_obj *obj, pdf_obj *res, fz_cookie *cookie, pdf_obj **out_res);
+void pdf_process_contents(fz_context *ctx, pdf_processor *proc, pdf_document *doc, pdf_obj *res, pdf_obj *stm, fz_cookie *cookie, pdf_obj **out_res);
 void pdf_process_annot(fz_context *ctx, pdf_processor *proc, pdf_annot *annot, fz_cookie *cookie);
 void pdf_process_glyph(fz_context *ctx, pdf_processor *proc, pdf_document *doc, pdf_obj *resources, fz_buffer *contents);
 
@@ -408,7 +473,7 @@ void pdf_process_glyph(fz_context *ctx, pdf_processor *proc, pdf_document *doc,
 	Function to process a contents stream without handling the resources.
 	The caller is responsible for pushing/popping the resources.
 */
-void pdf_process_raw_contents(fz_context *ctx, pdf_processor *proc, pdf_document *doc, pdf_obj *rdb, pdf_obj *stmobj, fz_cookie *cookie);
+void pdf_process_raw_contents(fz_context *ctx, pdf_processor *proc, pdf_document *doc, pdf_obj *stmobj, fz_cookie *cookie);
 
 /* Text handling helper functions */
 typedef struct
@@ -418,6 +483,7 @@ typedef struct
 	float scale;
 	float leading;
 	pdf_font_desc *font;
+	fz_string *fontname;
 	float size;
 	int render;
 	float rise;
@@ -443,7 +509,7 @@ void pdf_tos_save(fz_context *ctx, pdf_text_object_state *tos, fz_matrix save[2]
 void pdf_tos_restore(fz_context *ctx, pdf_text_object_state *tos, fz_matrix save[2]);
 fz_text *pdf_tos_get_text(fz_context *ctx, pdf_text_object_state *tos);
 void pdf_tos_reset(fz_context *ctx, pdf_text_object_state *tos, int render);
-int pdf_tos_make_trm(fz_context *ctx, pdf_text_object_state *tos, pdf_text_state *text, pdf_font_desc *fontdesc, int cid, fz_matrix *trm);
+int pdf_tos_make_trm(fz_context *ctx, pdf_text_object_state *tos, pdf_text_state *text, pdf_font_desc *fontdesc, int cid, fz_matrix *trm, float *adv);
 void pdf_tos_move_after_char(fz_context *ctx, pdf_text_object_state *tos);
 void pdf_tos_translate(pdf_text_object_state *tos, float tx, float ty);
 void pdf_tos_set_matrix(pdf_text_object_state *tos, float a, float b, float c, float d, float e, float f);
